@@ -18,11 +18,10 @@ from typing import Union
 from typing import Callable
 from typing import List
 from typing import Mapping
-from typing import TextIO
 from typing import Iterator
 from typing import Sequence
-from typing import Protocol
 from typing import Optional
+from typing import TYPE_CHECKING
 
 from liquid.exceptions import NoSuchFilterFunc
 from liquid.exceptions import ContextDepthError
@@ -31,6 +30,10 @@ from liquid.exceptions import lookup_warning
 
 from liquid import Mode
 
+if TYPE_CHECKING:
+    from liquid import Environment
+    from liquid import Template
+
 # Maximum number of times a context can be extended or wrapped.
 MAX_CONTEXT_DEPTH = 30
 
@@ -41,10 +44,10 @@ Namespace = Mapping[str, object]
 class ReadOnlyChainMap(collections.abc.Mapping):
     """Combine multiple mappings for sequential lookup.
 
-    For example, to emulate Python's normal lookup sequence:
+    Based on the "A greatly simplified read-only version of Chainmap" recipe link to
+    from https://docs.python.org/3/library/collections.html#chainmap-examples-and-recipes.
 
-        import __builtin__
-        pylookup = Chainmap(locals(), globals(), vars(__builtin__))
+    The link seems to be broken.
     """
 
     def __init__(self, *maps: Mapping[str, object]):
@@ -106,40 +109,6 @@ class BuiltIn(collections.abc.Mapping):
 builtin = BuiltIn()
 
 
-class Template(Protocol):
-    """Interface for delegating `get_template` to the environment."""
-
-    name: str
-
-    def render_with_context(
-        self: Any,
-        context: Context,
-        buffer: TextIO,
-        partial: bool = ...,
-        block_scope: bool = ...,
-    ):
-        ...
-
-
-# pylint: disable=too-few-public-methods
-class Env(Protocol):
-    """Interface for a Contexts' "env" argument.
-
-    For the benefit of the static type checker while avoiding cyclic imports.
-    """
-
-    mode: Mode
-    filters: Mapping[str, Callable[..., Any]]
-
-    # pylint: disable=redefined-builtin
-    def get_template(
-        self: Any,
-        name: str,
-        globals: Optional[Mapping[str, object]] = ...,
-    ) -> Template:
-        ...
-
-
 # pylint: disable=too-many-instance-attributes redefined-builtin
 class Context:
     """Liquid template context."""
@@ -149,14 +118,13 @@ class Context:
         "locals",
         "globals",
         "scope",
-        "cycles",
-        "counters",
+        "_tag_namespace",
         "disabled_tags",
     )
 
     def __init__(
         self,
-        env: Env,
+        env: Environment,
         globals: Optional[Namespace] = None,
         disabled_tags: Optional[List[str]] = None,
     ):
@@ -174,12 +142,13 @@ class Context:
         # temporary namespace is pushed to the front of this chain.
         self.scope = ReadOnlyChainMap(self.locals, self.globals, builtin)
 
-        # A distinct namespace for iterables created by the built-in "cycle" tag.
-        self.cycles = {}
-
-        # A namespace for integer counters. The built-in increment and decrement
-        # tags share this namespace.
-        self.counters = {}
+        # A namspace supporting stateful tags. Such as `cycle`, `increment`,
+        # `decrement` and `ifchanged`.
+        self._tag_namespace = {
+            "cycles": {},
+            "counters": {},
+            "ifchanged": "",
+        }
 
         # A list of tags names that are disallowed in this context. For example,
         # partial templates rendered using the "render" tag are not allowed to
@@ -205,13 +174,18 @@ class Context:
         return obj
 
     def resolve(self, name: str, default: object = None) -> Any:
+        """Return the object/value at `name` in the current scope.
+
+        This is like `get`, but does a single, top-level lookup rather than a
+        chained lookup from sequence of keys.
+        """
         try:
             return self.scope[name]
         except KeyError:
             # TODO: Resolve mode.
             return default
 
-    @functools.lru_cache
+    @functools.lru_cache(maxsize=128)
     def filter(self, name: str) -> Callable[..., object]:
         """Return the filter function with given name."""
         try:
@@ -230,14 +204,14 @@ class Context:
 
     def increment(self, name: str) -> int:
         """Increment the named counter and return it's value."""
-        val = self.counters.get(name, -1) + 1
-        self.counters[name] = val
+        val = self._tag_namespace["counters"].get(name, -1) + 1
+        self._tag_namespace["counters"][name] = val
         return val
 
     def decrement(self, name: str) -> int:
         """Decrement the named counter and return it's value."""
-        val = self.counters.get(name, 0) - 1
-        self.counters[name] = val
+        val = self._tag_namespace["counters"].get(name, 0) - 1
+        self._tag_namespace["counters"][name] = val
         return val
 
     def cycle(self, group_name: str, args: Sequence[Any]) -> Iterator[Any]:
@@ -245,9 +219,17 @@ class Context:
         if this is is the first time we're seeing this combination of group name
         and arguments."""
         key = f"{group_name}:{''.join([str(arg) for arg in args])}"
-        if key not in self.cycles:
-            self.cycles[key] = cycle(args)
-        return self.cycles[key]
+        if key not in self._tag_namespace["cycles"]:
+            self._tag_namespace["cycles"][key] = cycle(args)
+        return self._tag_namespace["cycles"][key]
+
+    def ifchanged(self, val: str) -> bool:
+        """Return True if the `ifchanged` value has changed."""
+        if val != self._tag_namespace["ifchanged"]:
+            self._tag_namespace["ifchanged"] = val
+            return True
+
+        return False
 
     @contextmanager
     def extend(self, namespace: Namespace):
