@@ -30,6 +30,8 @@ from liquid.token import TOKEN_INTEGER
 from liquid.token import TOKEN_FLOAT
 from liquid.token import TOKEN_EMPTY
 from liquid.token import TOKEN_NIL
+from liquid.token import TOKEN_NULL
+from liquid.token import TOKEN_BLANK
 from liquid.token import TOKEN_NEGATIVE
 from liquid.token import TOKEN_TRUE
 from liquid.token import TOKEN_FALSE
@@ -95,18 +97,18 @@ class Parser:
         return root
 
     def parse_statement(self, stream: TokenStream) -> ast.Node:
-        if stream.current.test(TOKEN_STATEMENT):
+        if stream.current.type == TOKEN_STATEMENT:
             node = self.statement.get_node(stream)
-        elif stream.current.test(TOKEN_TAG):
+        elif stream.current.type == TOKEN_TAG:
             tag = self.tags.get(stream.current.value, self.illegal)
 
             if tag.block:
-                stream.balancing_stack.append(tag)
+                stream.balancing_stack.append(tag.end)
 
             node = tag.get_node(stream)
 
             # Tag parse functions can choose to return an IllegalNode.
-            if node.__class__ == "IllegalNode":
+            if isinstance(node, ast.IllegalNode):
                 raise LiquidSyntaxError(
                     f"unexpected tag '{node.token().value}'",
                     linenum=node.token().linenum,
@@ -114,13 +116,12 @@ class Parser:
 
             if tag.block:
                 popped = stream.balancing_stack.pop()
-                if stream.current.value != popped.end:
+                if stream.current.value != popped:
                     raise LiquidSyntaxError(
-                        f"expected {popped.end}, found {stream.current.value}",
+                        f"expected {popped}, found {stream.current.value}",
                         linenum=stream.current.linenum,
                     )
         else:
-            expect(stream, TOKEN_LITERAL)
             node = self.literal.get_node(stream)
 
         return node
@@ -226,6 +227,11 @@ def parse_empty(_: TokenStream) -> expression.Empty:
     return expression.EMPTY
 
 
+def parse_blank(_: TokenStream) -> expression.Blank:
+    """Read a 'blank' keyword from the token stream."""
+    return expression.BLANK
+
+
 def parse_string_literal(stream: TokenStream) -> expression.StringLiteral:
     """Read a string from the token stream."""
     return expression.StringLiteral(value=stream.current.value)
@@ -440,7 +446,9 @@ class ExpressionParser:
             TOKEN_FALSE: parse_boolean,
             TOKEN_TRUE: parse_boolean,
             TOKEN_NIL: parse_nil,
+            TOKEN_NULL: parse_nil,
             TOKEN_EMPTY: parse_empty,
+            TOKEN_BLANK: parse_blank,
             TOKEN_NEGATIVE: self.parse_prefix_expression,
             TOKEN_STRING: parse_string_literal,
             TOKEN_INTEGER: parse_integer_literal,
